@@ -1,8 +1,9 @@
 fetch_pubmed_data <-
-function (pubmed_id_list, 
-                               retstart = 0, 
-                               retmax = 500,
-                               format = "xml") 
+  function (pubmed_id_list,
+            retstart = 0,
+            retmax = 500,
+            format = "xml", 
+            encoding = "UTF8") 
 {
   myIDlist <- pubmed_id_list
   if ((!is.list(myIDlist)) | is.na(myIDlist$WebEnv) | is.na(myIDlist$QueryKey) | 
@@ -40,12 +41,24 @@ function (pubmed_id_list,
                        "&retstart=", myRetstart, "&retmax=", myRetmax, 
                        "&rettype=", typeMode[1],"&retmode=", typeMode[2], 
                        sep = "")
-    tmpConnect <- url(efetch_url, open = "rb")
-    out.data <- readLines(tmpConnect, warn = FALSE, encoding = "UTF-8")
-    if (myFormat == "xml") {
-      out.data <- XML::xmlParse(out.data, encoding = "UTF-8")  
+    # api_key retrieval
+    api_key <- pubmed_id_list$APIkey
+    if (!is.null(api_key)) {
+      efetch_url <- paste(efetch_url, "&api_key=", api_key, sep = "")
     }
-    close.connection(tmpConnect)
+    
+    # Connect and data retrieval
+    tmpConnect <- url(efetch_url, open = "rb", encoding = "UTF8")
+    on.exit(close(tmpConnect))
+    out.data <- suppressWarnings(readLines(tmpConnect, warn = FALSE, encoding = "UTF8")) 
+    
+    if (encoding != "UTF8")
+      out.data <- base::iconv(out.data, from = "UTF8", to = encoding, sub = ".")
+
+    if (format[1] == "xml") {
+      out.data <- paste(out.data, collapse = "")
+    }
+                          
     return(out.data)
   }
 }
